﻿import { DataFilterState, DataFilterOptions, DataSpecs, DataFilterResponse, DataFilterAction } from "./models";
import { loop, Cmd, Loop } from "redux-loop";

const createApiCaller = <TFilterMap, TDomain, TK extends keyof TFilterMap = keyof TFilterMap>(filterName: TK) => {

    type A = DataFilterAction<TFilterMap, TDomain>;
    type FilterState = DataFilterState<TFilterMap[keyof TFilterMap]>;
    type FilterStateOrLoop = FilterState | Loop<FilterState, A>;
    
    return (state: DataFilterState<TFilterMap[TK]>,
        request: DataSpecs<TFilterMap[TK]>,
        
        options: DataFilterOptions<TFilterMap[TK], TDomain>): FilterStateOrLoop => 
            loop({
                ...state,
                error: null,
                isLoading: true,
                lastRequest: request
            }, Cmd.run(options.searchApi!, {
                args: [request],
                successActionCreator: (rs: DataFilterResponse<TDomain>) => ({
                    type: "FILTER/REQUEST_SUCCESS",
                    filterName,
                    request,
                    payload: rs
                } as A),
                failActionCreator: (err: any) => ({
                    type: "FILTER/REQUEST_ERROR",
                    filterName,
                    request,
                    payload: err
                } as A)
            }));
}

export default createApiCaller;