﻿import { useMappedState } from "redux-react-hook";
import { DataStateSelector } from "./models";

type SnapshotsByKey<TEntity> = { [key:string]: TEntity }

const none = { };

const getSnapshots = <TState, TFilterMap extends object, TDomain extends object, TK extends keyof TDomain>
    (stateSelector: DataStateSelector<TState, TFilterMap, TDomain>, entityName: TK) =>
        (state: TState): SnapshotsByKey<TDomain[TK]> | undefined =>
            stateSelector(state).entityCache[entityName];

type Denormalizer<TEntity> = 
    (key: string) => Partial<TEntity>;

const createDenormalizer = <TEntity>
    (snapshots: SnapshotsByKey<TEntity> | undefined): Denormalizer<TEntity> =>
        (key: string) => (snapshots || none as any)[key] || {};

export default
    <TState,TFilterMap extends object, TDomain extends object>
        (stateSelector: DataStateSelector<TState, TFilterMap, TDomain>) =>
        <TK extends keyof TDomain>
            (entityName: TK) => {
            const snapshots = useMappedState(getSnapshots<TState,TFilterMap,TDomain,TK>(stateSelector, entityName));
            return createDenormalizer(snapshots);
        };