﻿import { useEffect, useCallback } from "react";
import { Dispatch } from "redux";
import { useDispatch, useMappedState } from 'redux-react-hook';
import { DataFilterOptions, DataFilterState, DataFilterContainerActions, DataFilterAction, DataStateSelector } from "./models";
import { defaultFilterState } from "./defaults";

const none = {};

function getValue(e: any) {
    return (e && e.target) ? e.target.value : e;
}

export default <TState, TFilterMap extends object, TDomain extends object>
    (stateSelector: DataStateSelector<TState, TFilterMap, TDomain>) =>
    <TK extends keyof TFilterMap>
    (filterName:TK, { initialQuery, searchApi }: Partial<DataFilterOptions<TFilterMap[TK],TDomain>>) => {

        type TInput = TFilterMap[TK];

        const mappedState = useCallback((s: TState): DataFilterState<TInput> | undefined => 
            stateSelector(s).mounted[filterName], [filterName]);

        const m = useMappedState(mappedState);
        
        const dispatch: Dispatch<DataFilterAction<TFilterMap,TDomain>> = useDispatch();

        useEffect(() => {
            dispatch({
                type: "FILTER/MOUNT",
                filterName,
                options: { initialQuery }
            } as DataFilterAction<TFilterMap,TDomain>);
        
            return () => {
                dispatch({ type: "FILTER/UNMOUNT", filterName });
            };

        }, [filterName]);

        const getFilterValue = <TF extends keyof TInput, TDefault extends (TInput[TF] | undefined)>
            (name: TF, defaultValue?: TDefault) =>
                m? (m.lastRequest
                    ? m.lastRequest.input[name] || defaultValue
                    : defaultValue) : defaultValue;

        const actions: DataFilterContainerActions<TInput> = {

            refresh: () => dispatch({
                type: "FILTER/REFRESH",
                filterName
            }),

            setInput: (changes) => dispatch({
                type: "FILTER/INPUT_CHANGED",
                changes,
                filterName
            }),

            setPageIndex: (index) => dispatch({
                type: "FILTER/PAGE_INDEX_CHANGED",
                index,
                filterName
            }),

            setPageSize: (size) => dispatch({
                type: "FILTER/PAGE_SIZE_CHANGED",
                size,
                filterName
            }),

            setSortBy: (path, descending) => dispatch({
                type: "FILTER/SORT_BY_CHANGED",
                path,
                descending,
                filterName
            })
        };

        return {
            ...(m || defaultFilterState),
            ...actions,
            getFilterValue,
            getFilterInputProps: <TF extends keyof TInput>(name: TF, defaultValue:TInput[TF]) => ({
                name,
                disabled: false,
                value: getFilterValue(name, defaultValue),
                onChange: (e: any) => actions.setInput({ [name]: getValue(e) } as any)
            })
        };
    }
